/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/mozIGeckoMediaPluginService.idl
 */

#ifndef __gen_mozIGeckoMediaPluginService_h__
#define __gen_mozIGeckoMediaPluginService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_nsIThread_h__
#include "nsIThread.h"
#endif

#ifndef __gen_nsIPrincipal_h__
#include "nsIPrincipal.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
#include "nsTArray.h"
#include "nsStringGlue.h"
class GMPVideoDecoderProxy;
class GMPVideoEncoderProxy;
class GMPVideoHost;

/* starting interface:    mozIGeckoMediaPluginService */
#define MOZIGECKOMEDIAPLUGINSERVICE_IID_STR "7cef50ca-7a0f-41f2-9560-47abf709f0d7"

#define MOZIGECKOMEDIAPLUGINSERVICE_IID \
  {0x7cef50ca, 0x7a0f, 0x41f2, \
    { 0x95, 0x60, 0x47, 0xab, 0xf7, 0x09, 0xf0, 0xd7 }}

class NS_NO_VTABLE mozIGeckoMediaPluginService : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(MOZIGECKOMEDIAPLUGINSERVICE_IID)

  /* readonly attribute nsIThread thread; */
  NS_IMETHOD GetThread(nsIThread * *aThread) = 0;

  /* [noscript] GMPVideoDecoderProxy getGMPVideoDecoder (in TagArray tags, [optional] in AString origin, out GMPVideoHost outVideoHost); */
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) = 0;

  /* [noscript] GMPVideoEncoderProxy getGMPVideoEncoder (in TagArray tags, [optional] in AString origin, out GMPVideoHost outVideoHost); */
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) = 0;

  /* void addPluginDirectory (in AString directory); */
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) = 0;

  /* void removePluginDirectory (in AString directory); */
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(mozIGeckoMediaPluginService, MOZIGECKOMEDIAPLUGINSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_MOZIGECKOMEDIAPLUGINSERVICE \
  NS_IMETHOD GetThread(nsIThread * *aThread); \
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval); \
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval); \
  NS_IMETHOD AddPluginDirectory(const nsAString & directory); \
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_MOZIGECKOMEDIAPLUGINSERVICE(_to) \
  NS_IMETHOD GetThread(nsIThread * *aThread) { return _to GetThread(aThread); } \
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) { return _to GetGMPVideoDecoder(tags, origin, outVideoHost, _retval); } \
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) { return _to GetGMPVideoEncoder(tags, origin, outVideoHost, _retval); } \
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) { return _to AddPluginDirectory(directory); } \
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) { return _to RemovePluginDirectory(directory); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_MOZIGECKOMEDIAPLUGINSERVICE(_to) \
  NS_IMETHOD GetThread(nsIThread * *aThread) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetThread(aThread); } \
  NS_IMETHOD GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGMPVideoDecoder(tags, origin, outVideoHost, _retval); } \
  NS_IMETHOD GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGMPVideoEncoder(tags, origin, outVideoHost, _retval); } \
  NS_IMETHOD AddPluginDirectory(const nsAString & directory) { return !_to ? NS_ERROR_NULL_POINTER : _to->AddPluginDirectory(directory); } \
  NS_IMETHOD RemovePluginDirectory(const nsAString & directory) { return !_to ? NS_ERROR_NULL_POINTER : _to->RemovePluginDirectory(directory); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class _MYCLASS_ : public mozIGeckoMediaPluginService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_MOZIGECKOMEDIAPLUGINSERVICE

  _MYCLASS_();

private:
  ~_MYCLASS_();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(_MYCLASS_, mozIGeckoMediaPluginService)

_MYCLASS_::_MYCLASS_()
{
  /* member initializers and constructor code */
}

_MYCLASS_::~_MYCLASS_()
{
  /* destructor code */
}

/* readonly attribute nsIThread thread; */
NS_IMETHODIMP _MYCLASS_::GetThread(nsIThread * *aThread)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] GMPVideoDecoderProxy getGMPVideoDecoder (in TagArray tags, [optional] in AString origin, out GMPVideoHost outVideoHost); */
NS_IMETHODIMP _MYCLASS_::GetGMPVideoDecoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoDecoderProxy **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] GMPVideoEncoderProxy getGMPVideoEncoder (in TagArray tags, [optional] in AString origin, out GMPVideoHost outVideoHost); */
NS_IMETHODIMP _MYCLASS_::GetGMPVideoEncoder(nsTArray<nsCString> *tags, const nsAString & origin, GMPVideoHost **outVideoHost, GMPVideoEncoderProxy **_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void addPluginDirectory (in AString directory); */
NS_IMETHODIMP _MYCLASS_::AddPluginDirectory(const nsAString & directory)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removePluginDirectory (in AString directory); */
NS_IMETHODIMP _MYCLASS_::RemovePluginDirectory(const nsAString & directory)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_mozIGeckoMediaPluginService_h__ */
